const session = require('express').Router();
const pool = require('../../database.js');
const helpers = require('../../helpers.js');
// One-time passcode and QR code generators
const speakeasy = require('speakeasy');
const qrcode = require('qrcode');

// POST session - Authenticate a user - step 1
session.post('/', async (request, response) => {
  let conn;
  try {
    response.set("Content-Type", "application/json");
    conn = await pool.getConnection();

    // First, determine if the user entered their email or social number
    var data;
    var sql;
    var emailSubmitted = (request.body.emailOrSocialNum.indexOf("@") != -1);
    // Prepare SQL statement and parameters accordingly
    if (emailSubmitted) {
      data = [ request.body.emailOrSocialNum ];
      sql = `CALL get_login_details_by_email(?)`;
    } else {
      const parsedSocialNum = helpers.parseSocialNum(request.body.emailOrSocialNum);
      data = [ parsedSocialNum.birthDate, parsedSocialNum.socialNum ];
      sql = `CALL get_login_details_by_social_num(?, ?)`;
    }

    // Run SQL statement - getting login details
    const result = await conn.query(sql, data);
    const publicIdExists = Object.values(result[0][0])[0]; // Public ID is either email or social num
    const userDetails = result[1][0]; // Contains user ID, user type, password, and whether 2FA is enabled
    var userID;
    var userType;
    var hashedPassword;
    var twoFactorEnabled;

    if (publicIdExists) { // There is a match in the database for the public ID
      const userPassword = request.body.password;
      userID = userDetails.user_id;
      userType = userDetails.user_type;
      hashedPassword = userDetails.password;
      twoFactorEnabled = userDetails.two_factor_enabled;
      if (!helpers.verifyHash(userPassword, hashedPassword)) {  // Passwords don't match
        response.json({"status":"PasswordWrong"});
        return;
      }
    } else {
      (emailSubmitted) ? response.json({"status":"EmailDoesNotExist"}) : response.json({"status":"SocialNumDoesNotExist"});
      return;
    }

    // Two factor authentication
    if (twoFactorEnabled) { // The user has already setup two factor authentication
      response.json({"status":"OK","twoFactorEnabled":true,"userID":userID,"userType":userType});
      return;
    }
    // First time setup - generate a new secret
    var genSecret = speakeasy.generateSecret();
    var userSecret = genSecret.base32;

    const data3 = [ userSecret, userID ];
    const sql3 = `UPDATE User SET two_factor_secret=? WHERE user_id=?`;
    await conn.query(sql3, data3); // Store the secret in the database, but don't set two_factor_enabled just yet

    // Generate a QR code from the secret and send it to the client to be displayed
    qrcode.toDataURL(genSecret.otpauth_url, (err, data_url) => {
      if (err) {
        response.json({"status":"ERR", "error":err});
        return;
      }
      response.json({"status":"OK","twoFactorEnabled":false,"qrCode":data_url,"textCode":userSecret,"userID":userID,"userType":userType});
    });
  } catch (err) {
    response.json({"status":"ERR", "error":err});
  } finally {
    if (conn) return conn.end();
  }
});

// POST session - Authenticate a user - step 2 (by user ID)
session.post('/:id', async (request, response) => {
  let conn;
  try {
    response.set("Content-Type", "application/json");
    conn = await pool.getConnection();

    // The user has submitted a TOTP-token from the client-side secret
    // Get the server-side secret
    const sql = `SELECT two_factor_secret FROM User WHERE user_id=?`;
    const row = await conn.query(sql, [ request.params.id ]);
    const secret = row[0].two_factor_secret;

    // Verify the user-submitted token against the server-side secret
    const userToken = request.body.token;
    const base32secret = secret;
    const verified = speakeasy.totp.verify({ secret: base32secret,
                                       encoding: 'base32',
                                       token: userToken });
    if (verified) { // Set flag in database to complete 2FA setup (no effect if already setup)
      const sql2 = `UPDATE User SET two_factor_enabled=1 WHERE user_id=?`;
      await conn.query(sql2, [ request.params.id ]);
      // Now we can login the user
      response.json({"status":"OK","userID":request.params.id});
    } else {
      response.json({"status":"AuthenticationFailed"});
    }

  } catch (err) {
    response.json({"status":"ERR", "error":err});
  } finally {
    if (conn) return conn.end();
  }
});

module.exports = session;
