const users = require('express').Router();
const pool = require('../../database.js');
const helpers = require('../../helpers.js');
// Password strength estimator
const zxcvbn = require('zxcvbn');

// POST users - Register a new user
users.post('/', async (request, response) => {
  let conn;
  try {
    response.set("Content-Type", "application/json");
    conn = await pool.getConnection();

    // Evaluate the password
    const password = request.body.password;
    const pwdResult = zxcvbn(password);
    if (pwdResult.score < 2) { // zxcvbn's scoring scale is 0-4, disallow scores 0 and 1
      response.json({"status":"PasswordInsecure"});
      return;
    }

    // Process these user-submitted fields, the rest are sent as is
    const hashedPassword = helpers.hashPassword(password);
    const parsedSocialNum = helpers.parseSocialNum(request.body.social_num);

    // All the database logic is in the stored procedure "create_user"
    const data = [
      hashedPassword,
      request.body.user_type,
      request.body.first_name,
      request.body.last_name,
      request.body.email,
      request.body.phone_num,
      parsedSocialNum.birthDate,
      parsedSocialNum.socialNum,
      request.body.code
    ];
    const sql = `CALL create_user(?, ?, ?, ?, ?, ?, ?, ?, ?)`;
    const row2 = await conn.query(sql, data);
    response.json({"status":"OK"});

  } catch (err) {
    const error = err.message;
    // Handle custom errors returned from the database procedure
    if (error.indexOf("EmailExists") != -1) {
     response.json({"status":"EmailExists"});
   } else if (error.indexOf("InvitationCodeWrong") != -1) {
     response.json({"status":"InvitationCodeWrong"});
   } else if (error.indexOf("SocialNumExists") != -1) {
     response.json({"status":"SocialNumExists"});
   } else if (error.indexOf("SQLState: 22007") != -1) {
     response.json({"status":"DateInvalid"});
   } else {
     response.json({"status":"ERR", "error":error});
   }
  } finally {
    if (conn) return conn.end();
  }
});

// PUT users - Update an athlete's status
users.put('/status', async (request, response) => {
  let conn;
  try {
    response.set("Content-Type", "application/json");
    conn = await pool.getConnection();

    const data = [ request.body.status, request.body.user_id ];
    const sql = `UPDATE AthleteStatus SET status=? WHERE athlete_id=?`;
    await conn.query(sql, data);

    response.json({"status":"OK"});

  } catch (err) {
    response.json({"status":"ERR", "error":err});
  } finally {
    if (conn) return conn.end();
  }
});

// GET users - Get information about athletes by health professional's user ID
users.get('/hp/:id', async (request, response) => {
  let conn;
  try {
    response.set("Content-Type", "application/json");
    conn = await pool.getConnection();

    // First, get all the team IDs the health professional has a contract with
    const dataTeams = [ request.params.id ];
    const sqlTeams = `SELECT t.team_id FROM Team AS t
                      INNER JOIN Contract AS c ON t.team_id=c.team_id
                      INNER JOIN HealthOrganization AS ho ON c.health_organization_id=ho.organization_id
                      INNER JOIN HealthProfessional AS hp ON ho.organization_id=hp.organization_id
                      WHERE hp.health_professional_id=?`;
    // Prepared statement to get all relevant athlete information per team ID
    const sqlAthletes = `SELECT u.user_id, t.name AS team, a.shirt_no, a.position, u.first_name, u.last_name, astat.status, astat.summary,
                         cd.email, cd.phone_num, DATE_FORMAT(cd.birthdate,'%e/%c/%Y') AS birthdate, cd.social_num FROM User AS u
                         INNER JOIN ContactDetails cd ON u.user_id=cd.user_id
                         INNER JOIN Athlete AS a ON u.user_id=a.athlete_id
                         INNER JOIN AthleteStatus AS astat ON a.athlete_id=astat.athlete_id
                         INNER JOIN Team AS t ON a.team_id=t.team_id
                         WHERE t.team_id=? ORDER BY astat.status DESC, u.last_name ASC`;

    const teamIds = await conn.query(sqlTeams , dataTeams);
    delete teamIds.meta;
    var teamArray = [];
    // For each team, get all its users and push the team onto the master teamArray
    for (i in teamIds) {
      var data = [ teamIds[i].team_id ];
      var team = await conn.query(sqlAthletes, data);
      delete team.meta;
      for (i in team) { // Check each athlete
        if (team[i].status == 4) { // Status 4 is internal only, setting a user-friendly display value
          team[i].status = '?';
        }
      }
      teamArray.push(team);
    };
    response.json({"status":"OK","teams":teamArray});

  } catch (err) {
    response.json({"status":"ERR", "error":err});
  } finally {
    if (conn) return conn.end();
  }
});

// GET users - Get profile information by athlete's user ID
users.get('/ath/:id', async (request, response) => {
  let conn;
  try {
    response.set("Content-Type", "application/json");
    conn = await pool.getConnection();

    const data = [ request.params.id ];
    // Information to be displayed on an athlete user's home page
    const sql = `SELECT t.name AS team, a.shirt_no, a.position, u.first_name, u.last_name, astat.status, astat.summary FROM User AS u
                 INNER JOIN Athlete AS a ON u.user_id=a.athlete_id
                 INNER JOIN AthleteStatus AS astat ON a.athlete_id=astat.athlete_id
                 INNER JOIN Team AS t ON a.team_id=t.team_id
                 WHERE user_id=?`;
    const row = await conn.query(sql, data);
    const user = row[0];
    delete user.meta;
    response.json({"status":"OK","user":user});

  } catch (err) {
    response.json({"status":"ERR", "error":err});
  } finally {
    if (conn) return conn.end();
  }
});

// GET users - Get information about athletes by team representative's user ID
users.get('/tr/:id', async (request, response) => {
  let conn;
  try {
    response.set("Content-Type", "application/json");
    conn = await pool.getConnection();


    response.json({"status":"OK"});

  } catch (err) {
    response.json({"status":"ERR", "error":err});
  } finally {
    if (conn) return conn.end();
  }
});

module.exports = users;
