import 'whatwg-fetch';  // Fetch polyfill for older browsers
/*
  Action creators for fetching resources on the server.
  In your routes, call:
  response.set("Content-Type", "application/json");
  response.json(< JSON data to send to client >);
  Any redirects/refreshes should be done on the client side.
*/

export const FETCH_BEGIN = 'FETCH_BEGIN';
export const FETCH_SUCCESS = 'FETCH_SUCCESS';
export const FETCH_FAILURE = 'FETCH_FAILURE';

/*
  Three synchronous action creators for telling the reducers the status of the fetch
*/

export const fetchBegin = (url, options) => {
  return {
    type: FETCH_BEGIN
  };
};

export const fetchSuccess = (data) => {
  return {
    type: FETCH_SUCCESS,
    payload: { data }
  };
};

export const fetchFailure = (error) => {
  return {
    type: FETCH_FAILURE,
    payload: { error }
  };
};

/*
  This is the action creator we call from the components. How to call:

  import { appurl } from '../helpers/url.js';

  const url = appurl + "/api/<resource>";
  const fetchOptions = {
    method: "<HTTP method>",
    headers: { "Content-Type": "application/json" },
    body: <optional form data in JSON>
  };

  store.dispatch(fetchResource(url, fetchOptions)).then(() => {
    const data = store.getState().resources.data;
    // data contains the JSON that was sent in the route, such as status.
    // Handle data here.
  });

  The component must be connected to the store and import this function.
*/
export function fetchResource(url, options) {
  return function(dispatch) {
    dispatch(fetchBegin(url, options))
    return fetch(url, options)
    .then(
      response => response.json(),
      // Do not use catch, because that will also catch
      // any errors in the dispatch and resulting render,
      // causing a loop of 'Unexpected batch number' errors.
      // https://github.com/facebook/react/issues/6895
      error => dispatch(fetchFailure(error))
    )
    .then(json =>
      dispatch(fetchSuccess(json))
    )
  }
};
