import { LitElement, html, css } from 'lit-element';
import { SharedStyles } from './shared-styles.js';

import { connect } from 'pwa-helpers/connect-mixin.js';
import { store } from '../store.js';
import { appurl } from '../helpers/url.js';
import { fetchResource } from '../actions/server-fetch.js';

import './athlete-view.js';
import './health-prof-table.js';

/*
  Homepage for health professionals. Fetches information on athletes from the database
  and loads a HealthProfTable per team. Also receives events from the tables and loads
  athlete view pages or updates an athlete's status.
  Because of the static table width, this component is designed for a wide screen.
*/
class HealthProfHome extends connect(store)(LitElement) {
  static get properties() {
    return {
      userID: { type: Number },
      _teams: { type: Array },
      _viewingAthlete: { type: Boolean },
      _athlete: { type: Number }
    };
  }

  constructor() {
    super();
    this._teams = [];
    this._viewingAthlete = false;
  }

  static get styles() {
    return [
      SharedStyles,
      css`
      .page-wide-content-container {
        margin-top: 15px;
      }
      .table-container {
        width: 1040px;
        margin: 0 auto;
      }
      health-prof-table {
        margin-bottom: 24px;
      }
      `
    ];
  }

  render() {
    return html`
    <div class="page-wide-content-container">
      ${this._viewingAthlete ? html`
        <athlete-view .athlete="${this._athlete}" @backButtonClicked="${this._back}"></athlete-view>
      ` : html`
        <div class="table-container">
          ${this._teams.map((i) => html`
            <health-prof-table .team="${i}" @athleteButtonClicked="${this._viewAthlete}"
            @statusIndicatorClicked="${this._updateStatus}"></health-prof-table>
          `)}
        </div>
      `}
    </div>
    `
  }

  firstUpdated() {
    // Fetch teams with athletes
    const url = appurl + "/api/users/hp/" + this.userID;
    const fetchOptions = {
      method: "GET"
    };
    store.dispatch(fetchResource(url, fetchOptions)).then(() => {
      const teams = store.getState().resources.data.teams;
      this._teams = teams;
    });
  }

  // Switch to athlete view for the clicked athlete (in event)
  _viewAthlete(e) {
    this._athlete = e.detail;
    this._viewingAthlete = true;
  }

  // Go back to overview with tables; don't need to unset this._athlete
  _back() {
    this._viewingAthlete = false;
  }

  // Called from HealthProfTable. Update an athlete's status
  _updateStatus(e) {
    const athlete = e.detail; // An updated athlete object
    const url = appurl + "/api/users/status";
    const fetchOptions = {
      method: "PUT",
      headers: { "Content-Type": "application/json" },
      body: JSON.stringify(athlete)
    };
    // Update the athlete in the database
    store.dispatch(fetchResource(url, fetchOptions));
  }
}

window.customElements.define('health-prof-home', HealthProfHome);
