import { LitElement, html, css } from 'lit-element';
import { SharedStyles } from './shared-styles.js';
import '@vaadin/vaadin-grid/vaadin-grid.js';
import '@vaadin/vaadin-grid/vaadin-grid-column.js';
import '@vaadin/vaadin-grid/vaadin-grid-sort-column.js';
import '@vaadin/vaadin-button/vaadin-button.js';

/*
  Table for health professionals showing key information about the athletes on a team.
*/
class HealthProfTable extends LitElement {
  static get properties() {
    return {
      team: { type: Array }
    };
  }

  constructor() {
    super();
    this.team = [];
  }

  static get styles() {
    return [
      SharedStyles,
      css`
      /* Makes the two grids overlap so that there's only a 1px border between them */
      #summary-grid-container {
        transform: translateY(1px);
        -moz-transform: translateY(1px);
        -webkit-transform: translateY(1px);
        position: relative;
      }
      .summary-grid-overlay {
        width: 100%;
        height: 100%;
        position: absolute;
        cursor: pointer;
        z-index: 10;
      }
      .details {
        margin-right: 18px;
        margin-bottom: 4px;
        display: flex;
        justify-content: space-between;
        font-size: 18px;
      }
      .text-and-status, .status-box {
        display: flex;
        align-items: flex-start;
      }
      .status-box {
        position: relative;
        top: 7px;
        right: 10px;
      }
      .text {
        width: 200px;
      }
      .indicator {
        cursor: pointer;
      }
      .indicator:hover {
        background: #666;
        transition: background 0.1s linear;
        -moz-transition: background 0.1s linear;
        -webkit-transition: background 0.1s linear;
      }
      .button {
        width: 250px;
        display: flex;
        justify-content: flex-end;
      }
      `
    ];
  }

  render() {
    return html`
      <div id="summary-grid-container">
        <div class="summary-grid-overlay" @click="${this._toggleTable}"></div>
        <vaadin-grid id="summary-grid" theme="no-row-borders" height-by-rows>
          <vaadin-grid-column path="" width="170px" flex-grow="0"></vaadin-grid-column>
          <vaadin-grid-column path="team_name" header="Lag"></vaadin-grid-column>
          <vaadin-grid-column path="num_active" header="Antall spillere tilgjengelig" width="190px" flex-grow="0"></vaadin-grid-column>
          <vaadin-grid-column path="num_injured" header="Antall spillere ute" width="145px" flex-grow="0"></vaadin-grid-column>
          <vaadin-grid-column path="injury_rate" header="Skaderate i %" width="130px" flex-grow="0"></vaadin-grid-column>
          <vaadin-grid-column path="" width="170px" flex-grow="0"></vaadin-grid-column>
        </vaadin-grid>
      </div>
      <div id="main-grid-container" style="display:block;">
        <vaadin-grid id="main-grid" theme="row-stripes no-row-borders" height-by-rows>
          <vaadin-grid-sort-column path="first_name" header="Fornavn" width="170px" flex-grow="0"></vaadin-grid-sort-column>
          <vaadin-grid-sort-column path="last_name" header="Etternavn" width="210px" flex-grow="0"></vaadin-grid-sort-column>
          <vaadin-grid-sort-column path="status" header="Status" width="95px" flex-grow="0"></vaadin-grid-sort-column>
          <vaadin-grid-column path="summary" header="Merknad"></vaadin-grid-column>
        </vaadin-grid>
      </div>
    `;
  }

  firstUpdated() {
    const grid = this.shadowRoot.getElementById("main-grid");
    const summaryGrid = this.shadowRoot.getElementById("summary-grid");

    // Populate the grid with the team array passed in from HealthProfHome
    const team = this.team;
    grid.items = team;

    // Computes simple statistics based on the current team array, and populates summaryGrid with them
    function calculateStatistics() {
      var numActive = 0;
      var numInjured = 0;
      for (var i = 0; i<team.length; i++) {
        if (team[i].status === '0' || team[i].status === '1') {
          numActive++;
        } else if (team[i].status === '2' || team[i].status === '3') {
          numInjured++;
        }
      }
      var injuryRate = (numInjured/(numActive+numInjured)*100).toFixed(1);
      summaryGrid.items = [{'team_name': team[0].team, 'num_active': numActive, 'num_injured': numInjured, 'injury_rate': injuryRate}];
    }

    calculateStatistics();

    // Click event handler for the whole grid
    grid.addEventListener('click', (e) => {
      const context = grid.getEventContext(e);
      // Toggle row details
      if (context.section !== "details") { // The clicked area is not the details box
        if (grid.detailsOpenedItems.indexOf(context.item) === -1) { // Row details box is closed
          grid.openItemDetails(context.item);
        } else {
          grid.closeItemDetails(context.item);
          grid.notifyResize();
        }
      }
    });

    // Content of row details
    grid.rowDetailsRenderer = (root, grid, rowData) => {
      if (!root.firstElementChild) {
        root.innerHTML =
        '<div class="details">' +
          '<div class="text-and-status">' +
            '<div class="text"><span id="name"></span><br><small id="info"></small></div>' +
            '<div>' +
              '<small>Sett ny status:</small><br>' +
              '<div class="status-box">' +
                '<span class="indicator status-0">0</span>' +
                '<span class="indicator status-1">1</span>' +
                '<span class="indicator status-2">2</span>' +
                '<span class="indicator status-3">3</span>' +
                '<span class="indicator status-4">?</span>' +
              '</div>' +
            '</div>' +
          '</div>' +
          '<div class="button"><vaadin-button theme="primary">Spillerside</vaadin-button></div>' +
        '</div>';

        root.firstElementChild.querySelector('#name').textContent = rowData.item.first_name + " " + rowData.item.last_name;
        root.firstElementChild.querySelector('#info').innerHTML =
          rowData.item.birthdate + "<br>" + rowData.item.phone_num + "<br>" + rowData.item.email;

        if (rowData.item.shirt_no) {
          root.firstElementChild.querySelector('#info').innerHTML += "<br>Draktnr. " + rowData.item.shirt_no;
        }
        if (rowData.item.position) {
          root.firstElementChild.querySelector('#info').innerHTML += "<br>" + rowData.item.position;
        }
        // Button displays athlete page
        const button = root.firstElementChild.querySelector('vaadin-button');
        button.addEventListener('click', (e) => {
          this.dispatchEvent(new CustomEvent('athleteButtonClicked', {
            detail: rowData.item
          }));
        });

        // Five clickable status circles. Click one to change the athlete's status to it
        const indicator = root.firstElementChild.querySelectorAll('.indicator');
        indicator.forEach((currentObject, currentIndex, list) => {
          currentObject.addEventListener('click', (e) => {
            var newStatus = currentObject.textContent; // currentObject is a span class="status-X", newStatus is X or "?"
            if (newStatus === "?") newStatus = "4"; // Prepare for sending to the database
            rowData.item['status'] = newStatus;  // Update the athlete object
            this.dispatchEvent(new CustomEvent('statusIndicatorClicked', {  // This event triggers a server fetch
              detail: rowData.item  // Send the updated athlete object
            }));
            if (rowData.item['status'] === "4") rowData.item['status'] = "?"; // Set to user-friendly display value again
            // Update the grid
            calculateStatistics();
            grid.render();
          });
        });
      }
    };
  }

  // Toggles whether the table is displayed
  _toggleTable() {
    const table = this.shadowRoot.getElementById("main-grid-container");
    (table.style.display === "block") ? table.style.display = "none" : table.style.display = "block";
  }
}

window.customElements.define('health-prof-table', HealthProfTable);
