import { LitElement, html, css } from 'lit-element';

import { connect } from 'pwa-helpers/connect-mixin.js';
import { store } from '../store.js';
import { logIn } from '../actions/session.js';
import { fetchResource } from '../actions/server-fetch.js';

import { fixDrawerPosition } from '../helpers/position-fix.js';
import { formDataToJson } from '../helpers/form-data.js';
import { appurl } from '../helpers/url.js';
import { loginRequestPermission } from '../helpers/notification-permission.js';
import { eyeOnIcon, eyeOffIcon } from './my-icons.js';

import { SharedStyles } from './shared-styles.js';

/*
  The login component is displayed on the front page when logged out.
  It has two "subpages" (steps) that are rendered based on the _step property:
  basic email/password, and TOTP verification using a mobile app.
  The user is logged in once successfully completed step 2.
*/
class LogIn extends connect(store)(LitElement) {
  static get properties(){
    return {
      _drawerOpened: { type: Boolean },
      _step: { type: Number },
      userID: { type: Number },
      userType: { type: Number },
      twoFactorEnabled: { type: Boolean },
      imgUrl: { type: String },
      textCode: { type: String },
      eyeIcon: { type: String },
      authFailedMsg: { type: String },
      publicIdDoesNotExistMsg: { type: String },
      passwordWrongMsg: { type: String }
    };
  }

  constructor() {
    super();
    this._step = 1;
    this.eyeIcon = eyeOnIcon;
  }

  stateChanged(state) {
    this._drawerOpened = state.app.drawerOpened;
  }

  static get styles() {
    return [
      SharedStyles,
      css`
      .page-content-container {
        margin-top: 25px;
        margin-bottom: 20px;
      }
      .qr-container, .button-container {
        margin: 0 16px;
      }
      .qr-container p {
        line-height: 1.3em;
        margin: 0;
        padding: 0;
        text-align: left;
      }
      .qr-container p.intro {
        line-height: 1.5em;
      }
      p:last-of-type {
        margin-bottom: 5px;
      }
      .center {
        width: 100%;
        display: flex;
        justify-content: center;
      }
      textarea {
        width: 100%;
        color: black;
        font-size: 12px;
        padding: 2px;
        margin-top: 4px;
        resize: none;
      }
      .copy-button {
        font-size: 15px;
        text-align: center;
        background-color: var(--primary-color);
        color: var(--light-text-color);
        padding: 7px 30px;
        margin: 2px 0 10px 0;
        border-style: none;
        border-radius: 5px;
        -moz-border-radius: 5px;
        -webkit-border-radius: 5px;
      }
      .button-container {
        display: flex;
        flex-direction: row-reverse;
      }
      .flex-button {
        flex: 1;
        background-color: var(--primary-color);
        color: var(--light-text-color);
        font-size: 18px;
        padding: 15px 0;
        border-style: none;
        border-radius: 10px;
        -moz-border-radius: 10px;
        -webkit-border-radius: 10px;
      }
      .flex-button:first-child {
        margin-left: 3px;
      }
      .flex-button:last-child {
        margin-right: 3px;
      }
      @media (max-width: 479px) {
        .qr-container, .button-container {
          margin: 0 30px;
        }
        .button-container {
          flex-direction: column-reverse;
          align-items: center;
        }
        .flex-button {
          width: 285px;
        }
        .flex-button:first-child {
          margin-left: 0;
          margin-bottom: 4px;
        }
        .flex-button:last-child {
          margin-right: 0;
          margin-top: 4px;
        }
      }
      `
    ];
  }

  render() {
    fixDrawerPosition(this);
    return html`
      <div id="page" class="page-content-container">
        ${this._step === 1 ? html`
          <form id="loginForm" @submit="${this._submitFormStep1}">
            <div class="form-row">
              <div class="form-input">
                <label for="userPublicId">Epost eller personnummer</label>
                <input type="text" name="emailOrSocialNum" id="userPublicId" placeholder="Epost eller personnummer" required>
                <div class="input-notice-container">
                  <div class="input-notice">${this.publicIdDoesNotExistMsg}</div>
                </div>
              </div>
            </div>
            <div class="form-row">
              <div class="form-input">
                <label for="userPassword">Passord</label>
                <input type="password" name="password" id="userPassword" placeholder="Passord" required>
                <div class="input-notice-container">
                  <div class="input-notice-password">${this.passwordWrongMsg}</div>
                </div>
                <div class="input-eye-icon" @click="${this._eyeIconClicked}">${this.eyeIcon}</div>
              </div>
            </div>
            <input class="input-app-button" id="step1Button" type="submit" value="Neste">
          </form>
          <p><a href="/">Glemt passord</a></p>
          <p><a href="/nybruker">Registrer deg</a></p>

        ` : html`${this._step === 2 ? html`

          <div class="qr-container">
          ${!this.twoFactorEnabled ? html`
            <p class="intro">Nå skal vi sette opp tofaktor-autentisering. For det trenger du en tofaktor-app på mobilen.
            Vi anbefaler Google Autentisering, som er gratis og enkel å bruke.
            Hvis du er på PC eller nettbrett, skann denne QR-koden inn i en slik app:</p>
            <div class="center"><img src="${this.imgUrl}"></div>
            <p>Hvis du er på mobil, lim denne koden inn i appen:</p>
            <textarea id="textCode" rows="1" readonly>${this.textCode}</textarea>
            <div class="center"><button class="copy-button" id="copyButton" @click="${this.copy}">Kopier</button></div>
            <p>Skriv eller lim inn koden fra tofaktor-appen:</p>
            ` : html`
            <p>Skriv eller lim inn koden fra tofaktor-appen din:</p>
            `}
          </div>
          <form id="authForm" @submit="${this._submitFormStep2}">
            <div class="form-row">
              <div class="form-input">
                <label for="token">Kode</label>
                <input type="text" name="token" id="token" placeholder="Kode" autofocus required>
                <div class="input-notice-container">
                  <div class="input-notice">${this.authFailedMsg}</div>
                </div>
              </div>
            </div>
            <div class="button-container">
              <input class="flex-button" id="step2Button" type="submit" value="Logg inn">
              <button class="flex-button" id="backButton" @click="${this.back}">Tilbake</button>
            </div>
          </form>
        ` : html ``}`}
      </div>
    `
  }

  _submitFormStep1(event) {
    const formElement = this.shadowRoot.getElementById("loginForm");
    const jsonFormData = formDataToJson(formElement);
    const url = appurl + "/api/session";
    const fetchOptions = {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: jsonFormData
    };
    store.dispatch(fetchResource(url, fetchOptions)).then(() => {
      const data = store.getState().resources.data;
      // Reset any input field messages
      this.publicIdDoesNotExistMsg = "";
      this.passwordWrongMsg = "";
      switch (data.status) {
        case "OK":
          if (!data.twoFactorEnabled) {
            this.imgUrl = data.qrCode;
            this.textCode = data.textCode;
          }
          this.twoFactorEnabled = data.twoFactorEnabled;
          this.userID = data.userID;
          this.userType = data.userType;
          this.forward();
          break;
        case "EmailDoesNotExist":
          this.publicIdDoesNotExistMsg = "Epostadressen er ikke registrert.";
          break;
        case "SocialNumDoesNotExist":
          this.publicIdDoesNotExistMsg = "Personnummeret er ikke registrert.";
          break;
        case "PasswordWrong":
          this.passwordWrongMsg = "Feil passord";
          break;
        case "ERR":
          break;
        default:
          break;
      }
    });

    event.preventDefault();
  }

  _submitFormStep2(event) {
    const formElement = this.shadowRoot.getElementById("authForm");
    const jsonFormData = formDataToJson(formElement);
    const url = appurl + "/api/session/" + this.userID;
    const fetchOptions = {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: jsonFormData
    };
    store.dispatch(fetchResource(url, fetchOptions)).then(() => {
      const data = store.getState().resources.data;
      this.authFailedMsg = "";
      switch (data.status) {
        case "OK":
          store.dispatch(logIn(this.userID, this.userType));
          loginRequestPermission(this.userID);
          break;
        case "AuthenticationFailed":
          this.authFailedMsg = "Feil, vennligst prøv igjen."
          break;
        case "ERR":
          break;
        default:
          break;
      }
    });

    event.preventDefault();
  }

  _eyeIconClicked() {
    this.eyeIcon = (this.eyeIcon == eyeOnIcon) ? eyeOffIcon : eyeOnIcon;
    const input = this.shadowRoot.getElementById('userPassword');
    (input.getAttribute("type") == "password") ? input.setAttribute("type", "text") : input.setAttribute("type", "password");
  }

  // Shows the view for step 2
  forward() {
    this._step = 2;
  }

  // Shows the view for step 1
  back() {
    this._step = 1;
  }

  // Copies the contents of #textCode to clipboard
  copy() {
    const textArea = this.shadowRoot.getElementById("textCode");
    textArea.select();
    document.execCommand("copy");
  }

  firstUpdated() {
    // Hide the copy button if execCommand("copy") is not supported in the browser
    const copyButton = this.shadowRoot.getElementById("copyButton");
    if (!document.queryCommandSupported("copy")) {
      copyButton.style.display = "none";
    }
  }
}

window.customElements.define('log-in', LogIn);
