/**
@license
Copyright (c) 2018 The Polymer Project Authors. All rights reserved.
This code may only be used under the BSD style license found at http://polymer.github.io/LICENSE.txt
The complete set of authors may be found at http://polymer.github.io/AUTHORS.txt
The complete set of contributors may be found at http://polymer.github.io/CONTRIBUTORS.txt
Code distributed by Google as part of the polymer project is also
subject to an additional IP rights grant found at http://polymer.github.io/PATENTS.txt
*/

import 'promise-polyfill/src/polyfill';
import  { LitElement, html, css } from 'lit-element';
import { setPassiveTouchGestures } from '@polymer/polymer/lib/utils/settings.js';
import { connect } from 'pwa-helpers/connect-mixin.js';
import { installMediaQueryWatcher } from 'pwa-helpers/media-query.js';
import { installOfflineWatcher } from 'pwa-helpers/network.js';
import { installRouter } from 'pwa-helpers/router.js';
import { updateMetadata } from 'pwa-helpers/metadata.js';

// This element is connected to the Redux store.
import { store } from '../store.js';

// These are the actions and reducers needed by this element.
import {
  navigate,
  updateOffline,
  updateDrawerState
} from '../actions/app.js';
import { newIncomingMessage, addMessage } from '../actions/inbox.js';
import inbox from '../reducers/inbox.js';
import { logOut } from '../actions/session.js';
import session from '../reducers/session.js';
import resources from '../reducers/server-fetch.js';
import { appurl } from '../helpers/url.js';
store.addReducers({
  session, resources, inbox
});

// These are the elements needed by this element.
import '@polymer/app-layout/app-drawer/app-drawer.js';
import '@polymer/app-layout/app-header/app-header.js';
import '@polymer/app-layout/app-scroll-effects/effects/waterfall.js';
import '@polymer/app-layout/app-toolbar/app-toolbar.js';
import { menuIcon, profileIcon, logOutIcon, homeIcon } from './my-icons.js';
import './snack-bar.js';

class MyApp extends connect(store)(LitElement) {
  static get properties() {
    return {
      appTitle: { type: String },
      footerContent: { type: String },
      _page: { type: String },
      _pageTitle: { type: String },
      _drawerOpened: { type: Boolean },
      _snackbarOpened: { type: Boolean },
      _offline: { type: Boolean },
      _loggedIn: { type: Boolean },
      _userId: { type: Number },
      _userType: { type: Number },
      _newMessages: { type: Number }
    };
  }

  static get styles() {
    return [
      css`
        :host {
          display: block;

          --app-drawer-width: 256px;
          --app-drawer-scrim-background: rgba(0, 0, 0, 0);

          --primary-color: #00a1df;
          --secondary-color: #00a7a0;
          --tertiary-color: #1943b3;
          --text-color: #4f4f4f;
          --light-text-color: white;
          --light-sand-color: #f4f2ea;
          --medium-sand-color: #e4dfce;
          --dark-sand-color: #c6c5be;
          --red-reject-color: #c32534;

          --header-background-color: var(--primary-color);
          --header-text-color: white;
          --header-selected-color: white;

          --drawer-background-color: var(--tertiary-color);
          --drawer-text-color: var(--light-text-color);
          --drawer-selected-color: var(--primary-color);

          --status-0-color: #65db4e;
          --status-1-color: #ffea3c;
          --status-2-color: #ff9d24;
          --status-3-color: #db4646;
          --status-4-color: #999;

          --footer-margin: 30px;
          --footer-height: 64px;
        }

        a:link, a:visited, a:hover, a:active {
          text-decoration: none;
          color: var(--header-text-color);
        }

        app-header {
          position: fixed;
          z-index: 5;
          top: 0;
          left: 0;
          width: 100vw;
          text-align: center;
          background-color: var(--header-background-color);
          color: var(--header-text-color);
        }

        .toolbar-top {
          background-color: var(--header-background-color);
          position: relative;
          padding: 0;
        }

        [main-title] {
          font-family: 'Do Hyeon';
          font-size: 30px;
        }

        .main-title {
          width: 300px;
        }

        .header-tools-container {
          position: absolute;
          width: 100%;
          height: 100%;
          display: flex;
          align-items: center;
          justify-content: space-between;
          padding: 0 12px;
          box-sizing: border-box;
        }

        .toolbar-list {
          display: none;
        }

        .toolbar-list > a {
          display: inline-block;
          color: var(--header-text-color);
          text-decoration: none;
          line-height: 30px;
          padding: 4px 15px;
        }

        .toolbar-list > a[selected] {
          color: white; /*var(--header-selected-color);*/
          border-bottom: 4px solid var(--header-selected-color);
        }

        .drawer-btn, .right-btn {
          background: none;
          border: none;
          fill: var(--header-text-color);
          cursor: pointer;
          height: 44px;
          width: 44px;
        }

        .right-btn {
          line-height: 60px;
        }

        .drawer-list {
          box-sizing: border-box;
          width: 100%;
          height: 100%;
          padding: 24px;
          background: var(--drawer-background-color);
          color: var(--drawer-text-color);
          z-index: 10;
        }

        .drawer-list a {
          display: block;
          text-decoration: none;
          line-height: 40px;
          padding: 0 24px;
        }

        .drawer-list > a[selected] {
          color: var(--drawer-selected-color);

        }

        .drawer-footer {
          position: absolute;
          bottom: 150px;
        }

        /* Workaround for IE11 displaying <main> as inline */
        main {
          display: block;
        }

        /* Main content starts on the first pixel below the header.
           Therefore, each component needs to set its own margin-top. */
        .main-content {
          padding-top: 64px;
          /* Should be "calc(100vh - var(--footer-margin) - var(--footer-height))"
             but doesn't work in Firefox, so hardcoded instead. */
          min-height: calc(100vh - 94px);
          min-height: -moz-calc(100vh - 94px);
          min-height: -webkit-calc(100vh - 94px);
          box-sizing: border-box;
        }

        .page {
          display: none;
        }

        .page[active] {
          display: flex;
          flex-direction: row;
          justify-content: flex-start;
          align-items: stretch;
        }

        footer {
          width: 100vw;
          margin-top: var(--footer-margin);
          background: var(--primary-color);
          color: var(--drawer-text-color);
          height: var(--footer-height);
          display: none;
          align-items: center;
          justify-content: center;
        }

        .inline-icon {
          fill: var(--light-text-color);
          position: relative;
          top: 6px;
          left: 8px;
          z-index: -1;
        }

        .drawer-list .inline-icon {
          z-index: auto;
        }

        /* Wide layout: when the viewport width is bigger than 480px, layout
        changes to a wide layout */
        @media (min-width: 480px) {
          .toolbar-list {
            display: block;
          }

          .drawer-btn, .right-btn {
            visibility: hidden;
          }

          /* An invisible spacer that puts the main-content immediately below the header regardless of its height.
          Only shown when logged in, which is when toolbar-list is visible because there are links available. */
          .under-header-padding {
            width: 100vw;
            height: 50px;
          }

          footer {
            display: flex;
          }
        }

        @media (max-width: 340px) {
          [main-title] {
            font-size: 26px;
          }
        }
        .notifications-dot {
          display: inherit;
          /*position: relative;
          left: 58px;
          bottom: 22px;*/
          width: 18px;
          height: 18px;
          border-radius: 50%;
          -moz-border-radius: 50%;
          -webkit-border-radius: 50%;
          font-size: 11px;
          line-height: 19px;
          text-align: center;
          color: var(--light-text-color);
          background: var(--status-3-color);
        }
        .messages-link-box {
          /*display: inline-block;*/
          /*position: absolute;*/
        }
      `
    ];
  }

  // Called whenever properties update, such as _drawerOpened
  render() {
    // Like fixDrawerPosition(), but for the header
    const header = this.shadowRoot.getElementById('header');
    if (this._drawerOpened) {
      // Hide header behind drawer
      header.style.zIndex = 'auto';
    } else {
      // Wait for drawer closing animation before showing header again
      setTimeout(() => {
        try {
          header.style.zIndex = '5';
        } catch (err) {} // Suppress error when the header element is not found on first load
      }, 150);
    }

    return html`
      <!-- Header -->
      <app-header id="header" condenses reveals effects="waterfall">
        <app-toolbar class="toolbar-top">
          <div class="header-tools-container">
            <button class="drawer-btn" title="Meny" @click="${this._drawerButtonClicked}">${menuIcon}</button>
            <a class="main-title" title="Hjem" href="/"><div main-title>${this._pageTitle}</div></a>
            <a class="right-btn" title="Min profil" href="/">${this._loggedIn ? html`${profileIcon}` : html``}</a>
          </div>
        </app-toolbar>

        <!-- This gets hidden on a small screen-->
        <nav class="toolbar-list">
          ${this._loggedIn ? html`
          <a href="/">Hjem<span class="inline-icon">${homeIcon}</span></a>
          <a class="messages-link-box" ?selected="${this._page === 'meldinger'}" href="/meldinger">Meldinger ${this._newMessages > 0 ? html`<div class="notifications-dot">${this._newMessages}</div>`: html``}</a>
          <a @click="${this._logoutLinkClicked}" href="/">Logg ut<span class="inline-icon">${logOutIcon}</span></a>
          ` :
          html``}
        </nav>
      </app-header>

      <!-- Drawer content -->
      <app-drawer
          .opened="${this._drawerOpened}"
          @opened-changed="${this._drawerOpenedChanged}">
        <nav class="drawer-list">
          ${this._loggedIn ? html`
          <a class="messages-link-box" ?selected="${this._page === 'meldinger'}" href="/meldinger">Meldinger<div class="notification-dot">${this._newMessages}</div></a>
          <a @click="${this._logoutLinkClicked}" href="/">Logg ut<span class="inline-icon">${logOutIcon}</span></a>
          ` :
          html``}
          <div class="drawer-footer">
            <!-- This is the same content as in footer -->
            <a href="/">Personvernerklæring</a>
            <a href="/">Om appen</a>
          </div>
        </nav>
      </app-drawer>

      <!-- Main content -->
      <main role="main" class="main-content">
        <div class="under-header-padding" ?hidden="${this._loggedIn === false}"></div>
        <front-page class="page" ?active="${this._page === 'forside'}"></front-page>
        <new-user class="page" ?active="${this._page === 'nybruker'}"></new-user>
        <new-injury class="page" ?active="${this._page === 'registrerskade'}"></new-injury>
        <my-injuries class="page" ?active="${this._page === 'mineskader'}"></my-injuries>
        <inbox-overview class="page" ?active="${this._page === 'meldinger'}"></inbox-overview>
        <my-view404 class="page" ?active="${this._page === 'view404'}"></my-view404>
      </main>

      <footer>
        <p>Personvernerklæring &bull; Om appen</p>
      </footer>
      <snack-bar ?active="${this._snackbarOpened}">
        ${this._offline ? 'Ingen internett-tilkobling.' : 'Tilkoblet internett.'}
      </snack-bar>
    `;
  }

  constructor() {
    super();
    // To force all event listeners for gestures to be passive.
    // See https://www.polymer-project.org/3.0/docs/devguide/settings#setting-passive-touch-gestures
    setPassiveTouchGestures(true);
    // this.addToHomeScreen();
    if(firebase.messaging.isSupported()){
      const messaging = firebase.messaging();
      // Add event listener for Instance ID token refresh.
      // Currently, the refreshed token is stored in the database without overwriting the old,
      // because we don't know the old token.
      // Over time, this could lead to bloating in the database. Also, messages are sent to all
      // tokens connected to a user, which will be less efficient in the long run if there are
      // lots of obsolete tokens.
      messaging.onTokenRefresh(() => {
        messaging.getToken().then(refreshedToken => {
          console.log("Refreshed token: " + refreshedToken);
          // Store token in database
          const obj = { userID: this.userId, token: refreshedToken };
          const url = appurl + "/api/token";
          const fetchOptions = {
            method: "POST",
            headers: { "Content-Type": "application/json" },
            body: JSON.stringify(obj)
          };
          fetch(url, fetchOptions);
        }).catch((err) => {
          console.log('Unable to retrieve refreshed token ', err);
        });
      });

      // Foreground message handler
      messaging.onMessage(payload => {
        store.dispatch(addMessage({id: payload.data.id, from: payload.data.fromName, injury: payload.data.injury, athlete: payload.data.athlete, content: payload.data.content, date: payload.data.dateOfMessage}));
        if(this._userId !== Number(payload.data.fromUser)){
          // update number of new messages.
          store.dispatch(newIncomingMessage(payload.data.injury));
        }
     });
    }
  }

  firstUpdated() {
    installRouter((location) => this._locationChanged(location));
    installOfflineWatcher((offline) => store.dispatch(updateOffline(offline)));
    // Narrow layout ends at 479px, wide layout begins at 480px.
    installMediaQueryWatcher(`(min-width: 480px)`,
        () => store.dispatch(updateDrawerState(false)));
  }

  updated(changedProps) {
    if (changedProps.has('_page')) {
      this._pageTitle = this._page;

      switch(this._pageTitle) {
        case 'forside':
          this._pageTitle = 'Spillerstatus';
          break;
        case 'nybruker':
          this._pageTitle = 'Registrering';
          break;
        case 'mineskader':
          this._pageTitle = 'Mine skader';
          break;
        case 'registrerskade':
          this._pageTitle = 'Registrer ny skade'; // Title length should not be much longer than this,
          break;                                  // or it will overflow on narrow mobile screens
        case 'meldinger':
          this._pageTitle = 'Meldinger';
          break;
        case 'view404':
          this._pageTitle = '404';
          break;
        default:
      }

      updateMetadata({
        title: this._pageTitle,
        description: this._pageTitle
        // This object can also take an image property, that points to an img src.
      });
    }
  }

  _locationChanged(location) {
    store.dispatch(navigate(decodeURIComponent(location.pathname)));
  }

  _drawerButtonClicked() {
    store.dispatch(updateDrawerState(true));
  }

  _drawerOpenedChanged(e) {
    store.dispatch(updateDrawerState(e.target.opened));
  }

  _logoutLinkClicked() {
    store.dispatch(updateDrawerState(false));
    store.dispatch(logOut());
  }

  stateChanged(state) {
    this._page = state.app.page;
    this._offline = state.app.offline;
    this._snackbarOpened = state.app.snackbarOpened;
    this._drawerOpened = state.app.drawerOpened;
    this._loggedIn = state.session.loggedIn;
    this._userType = state.session.userType;
    this._userId = state.session.userID;
    this._newMessages = Number(state.inbox.newMessages);
  }

  // A2HS promt for the user to add the app to homescreen
  addToHomeScreen() {
    let installPromptEvent;


    this.window.addEventListener('beforeinstallprompt', (event) => {
      event.preventDefault();                                // Prevent Chrome 67 and earlier from automatically showing the prompt
      installPromptEvent = event;                            // Stash the event so it can be triggered later.
      document.querySelector('#install-button').disabled = false; // Update UI, notify the user they can add to home screen
    });

    this.btnInstall.addEventListener('click', () => {                // hide our user interface that shows our A2HS button
      document.querySelector('#install-button').disabled = true;  // Show the prompt
      installPromptEvent.prompt();                                // Wait for the user to respond to the prompt

      this.installPromptEvent.userChoice.then((choice) => {
        if (choice.outcome === 'accepted') {
          console.log('User accepted the A2HS prompt');
        } else {
          console.log('User dismissed the A2HS prompt');
        }

        installPromptEvent = null;        // Clear the saved prompt since it can't be used again
      });
    });

    this.window.addEventListener('appinstalled', (evt) => {  // Determine if the app was successfully installed
      app.logEvent('a2hs', 'installed');
    });
  }
}

window.customElements.define('my-app', MyApp);
